{-# LANGUAGE CPP #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-
	Copyright (C) 2012 Dr. Alistair Ward

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
-}
{- |
 [@AUTHOR@]	Dr. Alistair Ward

 [@DESCRIPTION@]	Implements 'Test.QuickCheck.Arbitrary' for 'Data.Array.IArray.Array'.

 [@EXAMPLE@]	@Test.QuickCheck.sample (Test.QuickCheck.arbitrary :: Test.QuickCheck.Gen.Gen (Data.Array.IArray.Array Char Int))@

 [@CAVEAT@]	An array with arbitrary indices isn't very useful.
-}

module ToolShed.Test.QuickCheck.Arbitrary.Array() where

import			Control.Applicative((<$>))
import qualified	Data.Array.IArray
import qualified	Test.QuickCheck

#if MIN_VERSION_base(4,3,0)
import	Data.Tuple(swap)
#else
-- | Swap the components of a pair.
swap :: (a, b) -> (b, a)
swap (a, b)	= (b, a)
#endif

instance (Data.Array.IArray.Ix i, Enum i, Test.QuickCheck.Arbitrary i, Test.QuickCheck.Arbitrary e) => Test.QuickCheck.Arbitrary (Data.Array.IArray.Array i e)	where
	arbitrary	= do
		pair	<- Test.QuickCheck.arbitrary {-(i, i)-}

		let
			bounds@(lower, upper)
				| uncurry (>) pair	= swap pair
				| otherwise		= pair

			indices	= [lower .. upper]

		Data.Array.IArray.array bounds . zip indices <$> Test.QuickCheck.vector (length indices)

#if !(MIN_VERSION_QuickCheck(2,1,0))
	coarbitrary	= undefined	--CAVEAT: stops warnings from ghc.
#endif

